@@ -1,10 +1,10 @@
 import io
 import os
 from http import HTTPStatus
+from typing import IO
 from typing import Any
 from typing import BinaryIO
 from typing import Dict
-from typing import IO
 from typing import List
 from typing import Optional
 from typing import Tuple
@@ -28,6 +28,9 @@ class IntezerApi:
     def on_premise_version(self) -> Optional[OnPremiseVersion]:
         return self.api.on_premise_version
 
+    def _get_tenant_id(self) -> Optional[str]:
+        return os.environ.get('INTEZER_TENANT_ID')
+
     def analyze_by_hash(self,
                         file_hash: str,
                         disable_dynamic_unpacking: Optional[bool],
@@ -263,30 +266,6 @@ class IntezerApi:
         raise_for_status(response)
         return response.json()['result']
 
-    def send_alert(self,
-                   alert: dict,
-                   definition_mapping: dict,
-                   **additional_parameters) -> str:
-        """
-        Send an alert for further investigation.
-
-        :param alert: The alert to send.
-        :param definition_mapping: The definition mapping that is used to extract relevant information from the alert.
-        :param additional_parameters: Additional parameters to pass to the API.
-
-        :raises: :class:`requests.HTTPError` if the request failed for any reason.
-        :return: The alert id of the submitted alert.
-        """
-        self.assert_any_on_premise('send-alert')
-        response = self.api.request_with_refresh_expired_access_token(method='POST',
-                                                                      path='/alerts/ingest',
-                                                                      data=dict(alert=alert,
-                                                                                definition_mapping=definition_mapping,
-                                                                                **additional_parameters))
-        raise_for_status(response, statuses_to_ignore=[HTTPStatus.BAD_REQUEST])
-        self._assert_alert_response_status_code(response)
-        return response.json()['alert_id']
-
     def send_binary_alert(self,
                           alert: io.BytesIO,
                           file_name: str,
@@ -531,6 +510,20 @@ class IntezerApi:
 
         return response.json()['result_url']
 
+    def get_code_reuse_by_code_block(self, sha256: str):
+        response = self.api.request_with_refresh_expired_access_token(
+            'POST', f'/files/{sha256}/code-reuse-by-code-block'
+        )
+
+        if response.status_code == HTTPStatus.NOT_FOUND:
+            raise errors.HashDoesNotExistError(response)
+        if response.status_code == HTTPStatus.CONFLICT:
+            raise ValueError('sha256 is not a code item')
+
+        raise_for_status(response)
+
+        return response.json()['result_url']
+
     def get_url_result(self, url: str) -> dict:
         """
         Send  GET request to an url.
@@ -699,6 +692,10 @@ class IntezerApi:
         data = dict(alert_ids=alert_ids)
         if environments:
             data['environments'] = environments
+        
+        if tenant_id := self._get_tenant_id():
+            data['tenant_id'] = tenant_id
+            
         response = self.api.request_with_refresh_expired_access_token(method='GET',
                                                                       path='/alerts/search',
                                                                       data=data)
@@ -718,6 +715,10 @@ class IntezerApi:
         data = dict(alert_id=alert_id)
         if environment:
             data['environment'] = environment
+            
+        if tenant_id := self._get_tenant_id():
+            data['tenant_id'] = tenant_id
+            
         response = self.api.request_with_refresh_expired_access_token(method='GET',
                                                                       path='/alerts/get-by-id',
                                                                       data=data)
@@ -726,8 +727,10 @@ class IntezerApi:
 
         return data_response['result'], data_response['status']
 
-    def get_incident_by_id(self, incident_id: str) -> dict:
+    def get_incident_by_id(self, incident_id: str, environment: Optional[str] = None) -> dict:
         data = dict(incident_id=incident_id)
+        if environment:
+            data['environment'] = environment
         response = self.api.request_with_refresh_expired_access_token(method='GET',
                                                                       path='/incidents/get-by-id',
                                                                       data=data)
@@ -746,6 +749,32 @@ class IntezerApi:
 
         return data_response['result']
 
+    def notify_alert(self, alert_id: str, environment: Optional[str] = None) -> dict:
+        """
+        Send a notification for an alert.
+
+        :param alert_id: The alert id to notify.
+        :param environment: The environment of the alert.
+        :raises: :class:`requests.HTTPError` if the request failed for any reason.
+        :return: The notification response containing notified_channels.
+        """
+        self.assert_any_on_premise('notify-alert')
+        
+        data = {}
+        if environment:
+            data['environment'] = environment
+            
+        if tenant_id := self._get_tenant_id():
+            data['tenant_id'] = tenant_id
+            
+        response = self.api.request_with_refresh_expired_access_token(
+            method='POST',
+            path=f'/alerts/{alert_id}/notify',
+            data=data if data else None
+        )
+        raise_for_status(response)
+        return response.json()
+
     def get_index_response(self, index_id: str) -> Response:
         """
         Get the index response by its id.
@@ -774,6 +803,52 @@ class IntezerApi:
 
         return self._get_analysis_id_from_response(response)
 
+    def get_raw_alert_data(
+        self,
+        alert_id: str,
+        environment: str,
+        raw_data_type: str = 'raw_alert',
+    ) -> dict:
+        """
+        Get raw alert data.
+
+        :param alert_id: The alert ID to get raw data for.
+        :param environment: The environment to get raw data from.
+        :param raw_data_type: The type of raw data to retrieve. Defaults to 'raw_alert'.
+        :return: The raw alert data.
+        """
+
+        data = {'environment': environment, 'raw_data_type': raw_data_type}
+        
+        if tenant_id := self._get_tenant_id():
+            data['tenant_id'] = tenant_id
+
+        response = self.api.request_with_refresh_expired_access_token(
+            method='GET',
+            path=f'/alerts/{alert_id}/raw-data',
+            data=data
+        )
+        raise_for_status(response)
+        return response.json()
+
+    def get_raw_incident_data(
+        self, incident_id: str, environment: str, raw_data_type: str = 'raw_incident'
+    ) -> dict:
+        """
+        Get raw incident data.
+
+        :param incident_id: The incident ID to get raw data for.
+        :param environment: The environment to get raw data from.
+        :param raw_data_type: The type of raw data to retrieve. Defaults to 'raw_incident'.
+        :return: The raw incident data.
+        """
+        data = {'environment': environment, 'raw_data_type': raw_data_type}
+        response = self.api.request_with_refresh_expired_access_token(
+            method='GET', path=f'/incidents/{incident_id}/raw-data', data=data
+        )
+        raise_for_status(response)
+        return response.json()
+
     @staticmethod
     def _assert_result_response(ignore_not_found: bool, response: Response):
         statuses_to_ignore = [HTTPStatus.NOT_FOUND] if ignore_not_found else None
