@@ -269,6 +269,7 @@ class Alert:
 
     def __init__(self,
                  alert_id: Optional[str] = None,
+                 environment: Optional[str] = None,
                  alert_stream: Optional[BinaryIO] = None,
                  api: IntezerApiClient = None):
         """
@@ -277,6 +278,7 @@ class Alert:
         instance with the given alert id.
 
         :param alert_id: The alert id.
+        :param environment: The environment of the alert.
         :param api: The API connection to Intezer.
         """
         if alert_stream and alert_id:
@@ -293,6 +295,7 @@ class Alert:
         else:
             self.alert_id: str = alert_id
 
+        self.environment = environment
         self._intezer_api_client = api
         self._api = IntezerApi(api or get_global_api())
         self._report: Optional[Dict] = None
@@ -319,12 +322,14 @@ class Alert:
 
         """
         try:
-            alert, status = self._api.get_alert_by_alert_id(alert_id=self.alert_id)
+            alert, status = self._api.get_alert_by_alert_id(alert_id=self.alert_id, environment=self.environment)
         except requests.HTTPError:
             self.status = AlertStatusCode.NOT_FOUND
             raise errors.AlertNotFoundError(self.alert_id)
 
         self._report = alert
+        if not self.environment:
+            self.environment = alert['environment']
 
         if status in (AlertStatusCode.IN_PROGRESS.value, AlertStatusCode.QUEUED.value):
             self.status = AlertStatusCode.IN_PROGRESS
@@ -358,6 +363,7 @@ class Alert:
     @classmethod
     def from_id(cls,
                 alert_id: str,
+                environment: Optional[str] = None,
                 api: IntezerApiClient = None,
                 fetch_scans: bool = False,
                 wait: bool = False,
@@ -366,6 +372,7 @@ class Alert:
         Create a new Alert instance, and fetch the alert data from the Intezer Analyze API.
 
         :param alert_id: The alert id.
+        :param environment: The environment of the alert.
         :param api: The API connection to Intezer.
         :param fetch_scans: Whether to fetch the scans for the alert - this could take some time.
         :param wait: Wait for the alert to finish processing before returning.
@@ -374,7 +381,7 @@ class Alert:
         :raises intezer_sdk.errors.AlertInProgressError: If the alert is still being processed.
         :return: The Alert instance, with the updated alert data.
         """
-        new_alert = cls(alert_id=alert_id, api=api)
+        new_alert = cls(alert_id=alert_id, environment=environment or None, api=api)
         status = new_alert.check_status()
         if status == AlertStatusCode.IN_PROGRESS and not wait:
             raise errors.AlertInProgressError(alert_id)
@@ -487,7 +494,7 @@ class Alert:
         send_alert_params = {key: value for key, value in send_alert_params.items() if value is not None}
         alert_id = _api.send_binary_alert(**send_alert_params)
 
-        alert = cls(alert_id=alert_id, api=api)
+        alert = cls(alert_id=alert_id, environment=environment, api=api)
         if wait:
             alert.wait_for_completion(timeout=timeout)
         return alert
@@ -547,3 +554,39 @@ class Alert:
                 _fetch_scan(scan, 'endpoint_analysis', EndpointAnalysis)
             elif scan_type == 'url':
                 _fetch_scan(scan, 'url_analysis', UrlAnalysis)
+
+    def get_raw_data(self,
+                     environment: Optional[str] = None,
+                     raw_data_type: str = 'raw_alert') -> dict:
+        """
+        Get raw alert data.
+
+        :param environment: The environment to get raw data from. If not provided, the environment will be taken from the alert.
+        :param raw_data_type: The type of raw data to retrieve. Defaults to 'raw_alert'.
+        :return: The raw alert data.
+        """
+        if not environment and not self.environment:
+            raise ValueError('Environment is required to get raw data.')
+
+        return self._api.get_raw_alert_data(
+            alert_id=self.alert_id,
+            environment=environment or self.environment,
+            raw_data_type=raw_data_type
+        )
+
+    def notify(self) -> List[str]:
+        """
+        Send a notification for this alert.
+
+        :raises intezer_sdk.errors.AlertNotFoundError: If the alert was not found.
+        :raises intezer_sdk.errors.AlertInProgressError: If the alert is still being processed.
+        :raises: :class:`requests.HTTPError` if the request failed for any reason.
+        :return: List of notified channels.
+        """
+        if self.status == AlertStatusCode.NOT_FOUND:
+            raise errors.AlertNotFoundError(self.alert_id)
+        elif self.status == AlertStatusCode.IN_PROGRESS:
+            raise errors.AlertInProgressError(self.alert_id)
+
+        response = self._api.notify_alert(self.alert_id, self.environment)
+        return response.get('notified_channels', [])
