@@ -1,4 +1,5 @@
 import datetime
+import logging
 import os
 from http import HTTPStatus
 from typing import Any
@@ -21,12 +22,18 @@ from intezer_sdk.consts import OnPremiseVersion
 
 _global_api: Optional['IntezerApi'] = None
 
+logger = logging.getLogger(__name__)
 
 def raise_for_status(response: requests.Response,
                      statuses_to_ignore: List[Union[HTTPStatus, int]] = None,
                      allowed_statuses: List[Union[HTTPStatus, int]] = None):
     """Raises stored :class:`HTTPError`, if one occurred."""
+    try:
+        response_json = response.json()
+    except Exception:
+        response_json = {}
 
+    should_raise = False
     http_error_msg = ''
     if isinstance(response.reason, bytes):
         reason = response.reason.decode('utf-8', 'ignore')
@@ -36,38 +43,27 @@ def raise_for_status(response: requests.Response,
     if statuses_to_ignore and response.status_code in statuses_to_ignore:
         return
     elif allowed_statuses and response.status_code not in allowed_statuses:
-        http_error_msg = f'{response.status_code} Custom Error: {reason} for url: {response.url}'
-    elif 400 <= response.status_code < 500:
+        should_raise = True
+    elif 400 <= response.status_code < 600:
+        should_raise = True
         if response.status_code == HTTPStatus.UNAUTHORIZED:
             raise errors.InvalidApiKeyError(response)
+        elif response.status_code == HTTPStatus.CONFLICT:
+            if response_json.get('result', {}).get('is_skipped_by_rule'):
+                raise errors.AnalysisSkippedByRuleError(response)
+        elif response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
+            raise errors.AnalysisRateLimitError(response)
         elif response.status_code == HTTPStatus.FORBIDDEN:
-            try:
-                error_message = response.json()['error']
-            except Exception:
-                http_error_msg = f'{response.status_code} Client Error: {reason} for url: {response.url}'
-            else:
-                if error_message == 'Insufficient Permissions':
-                    raise errors.InsufficientPermissionsError(response)
-        elif response.status_code != HTTPStatus.BAD_REQUEST:
+            if response_json.get('error') == 'Insufficient Permissions':
+                raise errors.InsufficientPermissionsError(response)
+        elif response.status_code == HTTPStatus.BAD_REQUEST:
+            http_error_msg = '\n'.join([f'{key}:{value}.' for key, value in response_json.get('message', {}).items()])
+
+    if should_raise:
+        if not http_error_msg:
             http_error_msg = f'{response.status_code} Client Error: {reason} for url: {response.url}'
         else:
-            # noinspection PyBroadException
-            try:
-                error = response.json()
-                http_error_msg = '\n'.join([f'{key}:{value}.' for key, value in error['message'].items()])
-            except Exception:
-                http_error_msg = f'{response.status_code} Client Error: {reason} for url: {response.url}'
-    elif 500 <= response.status_code < 600:
-        http_error_msg = f'{response.status_code} Server Error: {reason} for url: {response.url}'
-
-    if http_error_msg:
-        # noinspection PyBroadException
-        try:
-            data = response.json()
-            http_error_msg = f'{http_error_msg}, server returns {data["error"]}, details: {data.get("details")}'
-        except Exception:
-            pass
-
+            http_error_msg = f'{http_error_msg}, server returns {response_json.get("error")}, details: {response_json.get("details")}'
         raise requests.HTTPError(http_error_msg, response=response)
 
 
@@ -78,11 +74,15 @@ class IntezerApiClient:
                  api_key: str = None,
                  base_url: str = None,
                  verify_ssl: bool = True,
+                 proxies: Dict[str, str] = None,
                  on_premise_version: OnPremiseVersion = None,
                  user_agent: str = None,
-                 renew_token_window=20):
+                 renew_token_window=20,
+                 max_retry=3,
+                 timeout_in_seconds: Optional[int] = None):
         self.full_url = base_url + api_version
         self.base_url = base_url
+        self._proxies = proxies
         self.api_key = api_key
         self._access_token = None
         self._renew_token_window = renew_token_window
@@ -90,6 +90,8 @@ class IntezerApiClient:
         self._session = None
         self.verify_ssl = verify_ssl
         self.on_premise_version = on_premise_version
+        self.max_retry = max_retry
+        self.timeout_in_seconds = timeout_in_seconds
         if user_agent:
             user_agent = f'{consts.USER_AGENT}/{user_agent}'
         else:
@@ -103,7 +105,8 @@ class IntezerApiClient:
                  headers: dict = None,
                  files: dict = None,
                  stream: bool = None,
-                 base_url: str = None) -> Response:
+                 base_url: str = None,
+                 timeout_in_seconds: Optional[int] = None) -> Response:
         if not self._session:
             self._set_session()
 
@@ -116,16 +119,18 @@ class IntezerApiClient:
                 files=files,
                 data=data or {},
                 headers=headers or {},
-                stream=stream
+                stream=stream,
+                timeout=timeout_in_seconds or self.timeout_in_seconds
             )
-        elif isinstance(data, bytes):
+        elif isinstance(data, (bytes, str)):
             response = self._session.request(
                 method,
                 url,
                 files=files,
                 data=data,
                 headers=headers or {},
-                stream=stream
+                stream=stream,
+                timeout=timeout_in_seconds or self.timeout_in_seconds
             )
         else:
             response = self._session.request(
@@ -133,16 +138,16 @@ class IntezerApiClient:
                 url,
                 json=data or {},
                 headers=headers,
-                stream=stream
+                stream=stream,
+                timeout=timeout_in_seconds or self.timeout_in_seconds
             )
 
         return response
 
     def _refresh_token_if_needed(self):
         if self._token_expiration:
-            token_expire = datetime.datetime.fromtimestamp(self._token_expiration)
-            now = datetime.datetime.utcnow()
-            if (token_expire - now).total_seconds() < self._renew_token_window:
+            now = datetime.datetime.now()
+            if self._token_expiration - now.timestamp() < self._renew_token_window:
                 self._set_access_token()
 
     def request_with_refresh_expired_access_token(self,
@@ -152,21 +157,28 @@ class IntezerApiClient:
                                                   headers: dict = None,
                                                   files: dict = None,
                                                   stream: bool = None,
-                                                  base_url: str = None) -> Response:
-        self._refresh_token_if_needed()
-        response = self._request(method, path, data, headers, files, stream, base_url=base_url)
+                                                  base_url: str = None,
+                                                  timeout_in_seconds: Optional[int] = None) -> Response:
+        for retry_count in range(self.max_retry):
+            try:
+                self._refresh_token_if_needed()
+                response = self._request(method, path, data, headers, files, stream, base_url, timeout_in_seconds)
 
-        if response.status_code == HTTPStatus.UNAUTHORIZED and not self._token_expiration:
-            self._access_token = None
-            self._set_session()
-            response = self._request(method, path, data, headers, files, stream, base_url)
+                if response.status_code == HTTPStatus.UNAUTHORIZED:
+                    self._set_access_token()
+                    response = self._request(method, path, data, headers, files, stream, base_url, timeout_in_seconds)
 
-        return response
+                return response
+            except ConnectionError:
+                if self.max_retry - retry_count <= 1:
+                    raise
+                logger.warning('Encountered connection error, retrying', exc_info=True)
 
     def _set_access_token(self):
         response = requests.post(f'{self.full_url}/get-access-token',
                                  json={'api_key': self.api_key},
-                                 verify=self.verify_ssl)
+                                 verify=self.verify_ssl,
+                                 headers={'User-Agent': self.user_agent})
 
         if response.status_code in (HTTPStatus.UNAUTHORIZED, HTTPStatus.BAD_REQUEST):
             raise errors.InvalidApiKeyError(response)
@@ -174,8 +186,8 @@ class IntezerApiClient:
             raise_for_status(response)
 
         result = response.json()
-        self._access_token = result['result']
         self._token_expiration = result.get('expire_at')
+        self._session.headers['Authorization'] = f'Bearer {result["result"]}'
 
     def authenticate(self):
         """
@@ -186,13 +198,14 @@ class IntezerApiClient:
         self._set_session()
 
     def _set_session(self):
-        self._session = requests.session()
-        self._session.mount('https://', requests.adapters.HTTPAdapter(max_retries=3))
-        self._session.mount('http://', requests.adapters.HTTPAdapter(max_retries=3))
+        self._session = requests.sessions.Session()
+        self._session.mount('https://', requests.adapters.HTTPAdapter(max_retries=self.max_retry))
+        self._session.mount('http://', requests.adapters.HTTPAdapter(max_retries=self.max_retry))
         self._session.verify = self.verify_ssl
-        self._set_access_token()
-        self._session.headers['Authorization'] = f'Bearer {self._access_token}'
         self._session.headers['User-Agent'] = self.user_agent
+        if self._proxies:
+            self._session.proxies = self._proxies
+        self._set_access_token()
 
     def assert_on_premise_above_v21_11(self):
         if self.on_premise_version and self.on_premise_version <= OnPremiseVersion.V21_11:
@@ -206,6 +219,12 @@ class IntezerApiClient:
         if self.on_premise_version:
             raise errors.UnsupportedOnPremiseVersionError('This endpoint is not available yet on on-premise')
 
+    def is_available(self) -> bool:
+        response = requests.get(f'{self.full_url}/is-available')
+        if response.status_code == HTTPStatus.OK:
+            return response.json().get('is_available')
+        return False
+
 
 class IntezerApi(IntezerApiClient):
     def __init__(self,
@@ -214,13 +233,19 @@ class IntezerApi(IntezerApiClient):
                  base_url: str = None,
                  verify_ssl: bool = True,
                  on_premise_version: OnPremiseVersion = None,
-                 user_agent: str = None):
+                 user_agent: str = None,
+                 proxies: Dict[str, str] = None,
+                 timeout_in_seconds: Optional[int] = None,
+                 max_retry:int = 3):
         super().__init__(api_key=api_key,
                          base_url=base_url,
                          verify_ssl=verify_ssl,
                          user_agent=user_agent,
                          api_version=api_version,
-                         on_premise_version=on_premise_version)
+                         on_premise_version=on_premise_version,
+                         proxies=proxies,
+                         max_retry=max_retry,
+                         timeout_in_seconds=timeout_in_seconds)
 
     @deprecated('IntezerApi is deprecated and will be removed in the future')
     def analyze_by_hash(self,
@@ -228,10 +253,12 @@ class IntezerApi(IntezerApiClient):
                         disable_dynamic_unpacking: Optional[bool],
                         disable_static_unpacking: Optional[bool],
                         sandbox_command_line_arguments: str = None,
+                        sandbox_machine_type: str = None,
                         **additional_parameters) -> str:
         data = self._param_initialize(disable_dynamic_unpacking=disable_dynamic_unpacking,
                                       disable_static_unpacking=disable_static_unpacking,
                                       sandbox_command_line_arguments=sandbox_command_line_arguments,
+                                      sandbox_machine_type=sandbox_machine_type,
                                       **additional_parameters)
 
         data['hash'] = file_hash
@@ -248,12 +275,14 @@ class IntezerApi(IntezerApiClient):
                                 code_item_type: str = None,
                                 zip_password: str = None,
                                 sandbox_command_line_arguments: str = None,
+                                sandbox_machine_type: str = None,
                                 **additional_parameters) -> str:
         data = self._param_initialize(disable_dynamic_unpacking=disable_dynamic_unpacking,
                                       disable_static_unpacking=disable_static_unpacking,
                                       code_item_type=code_item_type,
                                       zip_password=zip_password,
                                       sandbox_command_line_arguments=sandbox_command_line_arguments,
+                                      sandbox_machine_type=sandbox_machine_type,
                                       **additional_parameters)
 
         data['download_url'] = download_url
@@ -284,12 +313,14 @@ class IntezerApi(IntezerApiClient):
                         code_item_type: str = None,
                         zip_password: str = None,
                         sandbox_command_line_arguments: str = None,
+                        sandbox_machine_type: str = None,
                         **additional_parameters) -> Optional[str]:
         options = self._param_initialize(disable_dynamic_unpacking=disable_dynamic_unpacking,
                                          disable_static_unpacking=disable_static_unpacking,
                                          code_item_type=code_item_type,
                                          zip_password=zip_password,
                                          sandbox_command_line_arguments=sandbox_command_line_arguments,
+                                         sandbox_machine_type=sandbox_machine_type,
                                          **additional_parameters)
 
         if file_stream:
@@ -609,16 +640,6 @@ class IntezerApi(IntezerApiClient):
 
         return self._get_analysis_id_from_response(response)
 
-    @deprecated('IntezerApi is deprecated and will be removed in the future')
-    def get_edr_assessments_by_alert_ids(self, edr_alert_ids: List[str]):
-        response = self.request_with_refresh_expired_access_token(path='/edr-connector/notes/get-by-edr-alert-ids',
-                                                                  method='GET',
-                                                                  data=dict(edr_alert_ids=edr_alert_ids))
-
-        raise_for_status(response)
-
-        return response.json()
-
     @staticmethod
     def _assert_result_response(ignore_not_found: bool, response: Response):
         statuses_to_ignore = [HTTPStatus.NOT_FOUND] if ignore_not_found else None
@@ -630,6 +651,7 @@ class IntezerApi(IntezerApiClient):
                           code_item_type: str = None,
                           zip_password: str = None,
                           sandbox_command_line_arguments: str = None,
+                          sandbox_machine_type: str = None,
                           **additional_parameters):
         data = {}
 
@@ -643,6 +665,8 @@ class IntezerApi(IntezerApiClient):
             data['zip_password'] = zip_password
         if sandbox_command_line_arguments:
             data['sandbox_command_line_arguments'] = sandbox_command_line_arguments
+        if sandbox_machine_type:
+            data['sandbox_machine_type'] = sandbox_machine_type
 
         data.update(additional_parameters)
 
@@ -653,6 +677,10 @@ class IntezerApi(IntezerApiClient):
         if response.status_code == HTTPStatus.NOT_FOUND:
             raise errors.HashDoesNotExistError(response)
         elif response.status_code == HTTPStatus.CONFLICT:
+            is_skipped_by_rule = response.json().get('result', {}).get('is_skipped_by_rule')
+            if is_skipped_by_rule:
+                raise errors.AnalysisSkippedByRuleError(response)
+
             running_analysis_id = response.json().get('result', {}).get('analysis_id')
             raise errors.AnalysisIsAlreadyRunningError(response, running_analysis_id)
         elif response.status_code == HTTPStatus.FORBIDDEN:
@@ -696,27 +724,42 @@ def get_global_api() -> IntezerApi:
 
 
 def set_global_api(api_key: str = None,
-                   api_version: str = consts.API_VERSION,
-                   base_url: str = consts.BASE_URL,
+                   api_version: str = None,
+                   base_url: str = None,
                    verify_ssl: bool = True,
-                   on_premise_version: OnPremiseVersion = None) -> IntezerApiClient:
+                   on_premise_version: OnPremiseVersion = None,
+                   proxies: Dict[str, str] = None) -> IntezerApiClient:
     """
     Configure the global api
 
     :param api_key: The api key
     :param api_version: The api version
     :param base_url: The base url. Configure this when using on-premise.
-    :param verify_ssl: Weather to verify ssl
+    :param verify_ssl: A requests compatible "verify" value. Setting as `False` will not verify the SSL certificate
     :param on_premise_version: You're on-premise version
+    :param proxies: A requests compatible "proxies" dict
     :return: The configured api
     """
     global _global_api
     api_key = api_key or os.environ.get('INTEZER_ANALYZE_API_KEY')
-    _global_api = IntezerApi(api_version,
-                             api_key,
-                             base_url,
-                             verify_ssl,
-                             on_premise_version)
+    _global_api = IntezerApi(api_version=api_version or consts.API_VERSION,
+                             api_key=api_key,
+                             base_url=base_url or consts.BASE_URL,
+                             verify_ssl=verify_ssl,
+                             on_premise_version=on_premise_version,
+                             proxies=proxies)
+    return _global_api
+
+
+def set_global_api_custom_instance(api: IntezerApiClient) -> IntezerApiClient:
+    """
+    Configure the global api with a custom instance
+
+    :param api: The api instance
+    :return: The configured api
+    """
+    global _global_api
+    _global_api = api
     return _global_api
 
 
